import codecs

import pytest

from . import v
assert v  # Silence pyflakes.


def test_syntax_error(v):
    v.scan("foo bar")
    assert int(v.report()) == 1


def test_null_byte(v):
    v.scan("\x00")
    assert int(v.report()) == 1


def test_confidence_range(v):
    v.scan("""\
def foo():
    pass
""")
    with pytest.raises(ValueError):
        v.get_unused_code(min_confidence=150)


def test_non_utf8_encoding(v, tmpdir):
    code = """\
def foo():
    pass

foo()
"""
    non_utf_8_file = str(tmpdir.mkdir("non_utf8").join("non_utf8.py"))
    with open(non_utf_8_file, 'wb') as f:
        f.write(codecs.BOM_UTF16_LE)
        f.write(code.encode('utf_16_le'))
    v.scavenge([f.name])
    assert v.found_dead_code_or_error



import pytest

from . import check, v
assert v  # silence pyflakes


@pytest.fixture
def check_whitelist(v, capsys):
    def examine(code, results_before, results_after):
        v.scan(code)
        check(v.get_unused_code(), results_before)
        capsys.readouterr()  # Clear captured text
        v.make_whitelist()
        whitelist = capsys.readouterr().out
        v.scan(whitelist)
        check(v.get_unused_code(), results_after)
    return examine


def test_unused_function(check_whitelist):
    code = """\
def func():
    pass
"""
    check_whitelist(code, ['func'], [])


def test_unused_class(check_whitelist):
    code = """\
class Foo:
    def __init__(self):
        pass
"""
    check_whitelist(code, ['Foo'], [])


def test_unused_variables(check_whitelist):
    code = """\
foo = 'unused'
bar = 'variable'
"""
    check_whitelist(code, ['foo', 'bar'], [])


def test_unused_import(check_whitelist):
    code = """\
import xyz
import foo as bar
from abc import iou
from lorem import ipsum as dolor
"""
    check_whitelist(code, ['xyz', 'bar', 'iou', 'dolor'], [])


def test_unused_attribute(check_whitelist):
    code = """\
class Foo:
    def bar(self):
        self.foobar = 'unused attr'
"""
    check_whitelist(code, ['Foo', 'bar', 'foobar'], [])


def test_unused_property(check_whitelist):
    code = """\
class Foo:
    @property
    def bar(self):
        pass
"""
    check_whitelist(code, ['Foo', 'bar'], [])


def test_unreachable_code(check_whitelist):
    code = """\
def foo():
    return "Foo Bar"
    print("Hello")
"""
    check_whitelist(code, ['foo', 'return'], ['return'])



from . import v
assert v  # Silence pyflakes


def test_sorting(v):
    v.scan("""\
def foo():
    print("Hello, I am a long function.")
    return "World"

def bar():
    pass
""")
    assert [item.name for item in
            v.get_unused_code(sort_by_size=True)] == ['bar', 'foo']
    assert [item.name for item in
            v.get_unused_code(sort_by_size=False)] == ['foo', 'bar']



from . import v
assert v  # Silence pyflakes


def test_item_repr(v):
    v.scan("""\
import os

message = "foobar"

class Foo:
    def bar():
        pass
""")
    for item in v.get_unused_code():
        assert repr(item) == "'{}'".format(item.name)



import os.path
import subprocess
import sys

from . import call_vulture, REPO, WHITELISTS


def test_module_with_explicit_whitelists():
    assert call_vulture(['vulture/'] + WHITELISTS) == 0


def test_module_with_implicit_whitelists():
    assert call_vulture(['vulture/']) == 0


def test_module_without_whitelists():
    assert call_vulture(['vulture/', '--exclude', 'whitelists']) == 1


def test_missing_file():
    assert call_vulture(['missing.py']) == 1


def test_tests():
    assert call_vulture(['tests/']) == 0


def test_whitelists_with_python():
    for whitelist in WHITELISTS:
        assert subprocess.call([sys.executable, whitelist], cwd=REPO) == 0


def test_pyc():
    assert call_vulture(['missing.pyc']) == 1


def test_sort_by_size():
    assert call_vulture(['vulture/utils.py', '--sort-by-size']) == 1


def test_min_confidence():
    assert call_vulture([
        'vulture/core.py', '--exclude', 'whitelists',
        '--min-confidence', '100']) == 0


def test_exclude():
    def get_csv(paths):
        return ",".join(os.path.join('vulture', path) for path in paths)

    def call_vulture_with_excludes(excludes):
        return call_vulture(['vulture/', '--exclude', get_csv(excludes)])

    assert call_vulture_with_excludes(['core.py', 'utils.py']) == 1
    assert call_vulture_with_excludes(['core.py', 'utils.py', 'lines.py']) == 0


def test_make_whitelist():
    assert call_vulture(
            ['vulture/', '--make-whitelist', '--exclude', 'whitelists']) == 1
    assert call_vulture(['vulture/', '--make-whitelist']) == 0



#! /usr/bin/env python

import codecs
import os.path
import re
import sys

import setuptools
from setuptools.command.test import test as TestCommand


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, *parts), 'r') as f:
        return f.read()


def find_version(*file_parts):
    version_file = read(*file_parts)
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]$", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)

        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest

        sys.exit(pytest.main(self.test_args))


setuptools.setup(
    name='vulture',
    version=find_version('vulture', 'core.py'),
    description='Find dead code',
    long_description='\n\n'.join(
        [open('README.rst').read(), open('NEWS.rst').read()]),
    keywords='dead-code-removal',
    author='Jendrik Seipp',
    author_email='jendrikseipp@gmail.com',
    url='https://github.com/jendrikseipp/vulture',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Quality Assurance'
    ],
    entry_points={
        'console_scripts': ['vulture = vulture.core:main'],
    },
    tests_require=['pytest', 'pytest-cov'],
    cmdclass={'test': PyTest},
    packages=setuptools.find_packages(exclude=['tests']),
    package_data={'vulture': ['whitelists/*.py']},
)
